<?php
/**
 * Plugin Name: Business Manager
 * Description: Business Manager is a HR, ERP, CRM, and Project Management plugin that allows you to easily manage your employees, projects, clients, documents and more.
 * Author: Business Manager
 * Author URI: https://bzmngr.com
 * Version: 1.4.6
 * Text Domain: 'business-manager'
 * Domain Path: languages
 * License: GPL2 or later.
 *
 */

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit;
}

/*
 * Helper function for quick debugging
 */
if (!function_exists('pp')) {
    function pp($array)
    {
        echo '<pre style="white-space:pre-wrap;">';
        print_r($array);
        echo '</pre>'."\n";
    }
}

/**
 * Main Class.
 *
 * @since 1.0.0
 */
final class Business_Manager
{
    /**
     * @var The one true instance
     * @since 1.0.0
     */
    protected static $_instance = null;

    public $version = '1.4.6';

    /**
     * Main Instance.
     */
    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * Throw error on object clone.
     *
     * @since 1.0.0
     * @access protected
     * @return void
     */
    public function __clone()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?', 'business-manager'), '1.0.0');
    }

    /**
     * Disable unserializing of the class.
     * @since 1.0.0
     */
    public function __wakeup()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?', 'business-manager'), '1.0.0');
    }

    /**
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        $this->define_constants();
        $this->includes();
        $this->localisation();
        $this->init();

        do_action('business_manager_loaded');
    }

    /**
     * Define Constants.
     * @since  1.0.0
     */
    private function define_constants()
    {
        $this->define('BUSINESSMANAGER_DIR', plugin_dir_path(__FILE__));
        $this->define('BUSINESSMANAGER_URL', plugin_dir_url(__FILE__));
        $this->define('BUSINESSMANAGER_BASENAME', plugin_basename(__FILE__));
        $this->define('BUSINESSMANAGER_VERSION', $this->version);
    }

    /**
     * Define constant if not already set.
     * @since  1.0.0
     */
    private function define($name, $value)
    {
        if (!defined($name)) {
            define($name, $value);
        }
    }

    /**
     * Include required files.
     * @since  1.0.0
     */
    public function includes()
    {
        // cmb
        include_once 'includes/lib/cmb2/init.php';
        include_once 'includes/lib/cmb2-tabs-master/plugin.php';
        include_once 'includes/lib/cmb2-attached-posts/cmb2-attached-posts-field.php';
        include_once 'includes/lib/cmb2-conditionals/cmb2-conditionals.php';

        // rrule
        include_once 'includes/lib/rrule/RRuleInterface.php';
        include_once 'includes/lib/rrule/RRule.php';
        include_once 'includes/lib/rrule/RfcParser.php';
        include_once 'includes/lib/rrule/RSet.php';

        // lib
        include_once 'includes/lib/class-columns.php';
        include_once 'includes/lib/class-settings-api.php';

        include_once 'includes/class-setup.php';
        include_once 'includes/class-post-types.php';
        include_once 'includes/class-menus.php';
        include_once 'includes/class-settings.php';

        include_once 'includes/functions.php';
        include_once 'includes/functions-metaboxes.php';
        include_once 'includes/functions-columns.php';

        // dashboard
        include_once 'includes/dashboard/class-dashboard.php';

        // project
        include_once 'includes/project/class-project.php';
        include_once 'includes/project/class-metaboxes.php';
        include_once 'includes/project/class-columns.php';

        // client
        include_once 'includes/client/class-client.php';
        include_once 'includes/client/class-metaboxes.php';
        include_once 'includes/client/class-columns.php';

        // employee
        include_once 'includes/employee/class-employee.php';
        include_once 'includes/employee/class-metaboxes.php';
        include_once 'includes/employee/class-columns.php';

        // leave
        include_once 'includes/leave/class-leave.php';
        include_once 'includes/leave/class-metaboxes.php';
        include_once 'includes/leave/class-columns.php';

        // review
        include_once 'includes/review/class-review.php';
        include_once 'includes/review/class-metaboxes.php';
        include_once 'includes/review/class-columns.php';

        // document
        include_once 'includes/document/class-document.php';
        include_once 'includes/document/class-metaboxes.php';
        include_once 'includes/document/class-columns.php';
    }

    public function init()
    {
        $this->settings = new Business_Manager_Settings();

        register_activation_hook(__FILE__, [$this, 'bm_activation_employee_post_titles']);

        $this->bm_roles();

        add_action('add_meta_boxes', [$this, 'bm_metabox_extension_upsell']);

        add_filter('admin_init', [$this, 'bm_redirect']);
        add_filter('admin_body_class', [$this, 'bm_body_class']);
    }

    /**
     * Create Business Manager roles for WordPress users.
     * @since  1.3.3
     */
    public function bm_activation_employee_post_titles()
    {
        $employees = new WP_Query([
            'post_type' => 'bm-employee',
            'post_status' => 'publish',
            'posts_per_page' => 999999,
        ]);

        while ($employees->have_posts()) {
            $employees->the_post();
            $employee = new Business_Manager_Employee(get_the_ID());
            $employee->update_title();
        }

        wp_reset_query();
    }

    public function bm_metabox_extension_upsell()
    {
        global $post;

        $bm_post_types = ['bm-employee', 'bm-leave', 'bm-review', 'bm-project', 'bm-client', 'bm-document', 'bm-asset', 'bm-contractor'];
        $post_type = get_post_type_object($post->post_type);

        $extensions = ['business-manager-asset-manager', 'business-manager-contractors', 'business-manager-custom-fields'];
        $missing_extensions = false;

        foreach ($extensions as $extension) {
            if (!is_plugin_active($extension.'/'.$extension.'.php')) {
                $missing_extensions = true;
                break;
            }
        }

        if ($missing_extensions === true && in_array($post->post_type, $bm_post_types)) {
            $bm_access = business_manager_employee_access(get_current_user_id(), 'bm_access_'.strtolower($post_type->label));

            if (current_user_can('manage_options') || $bm_access == 'full') {
                add_meta_box(
                    'business-manager-extension-upsell',
                    __('Extensions', 'business-manager'),
                    'business_manager_extension_upsell',
                    $bm_post_types,
                    'side',
                    'core',
                    ['bm_post_types' => $bm_post_types, 'post_type' => $post_type]
                );
            }
        }
    }

    /**
     * Create Business Manager roles for WordPress users.
     * @since  1.4.0
     */
    public function bm_roles()
    {
        $wp_roles = wp_roles();
        if (isset($wp_roles) && $wp_roles->is_role('bm_employee') === false) {
            add_role(
                'bm_employee',
                'Business Manager Employee',
                [
                    'read' => true,
                    'publish_posts' => true,
                    'edit_posts' => true,
                    'edit_others_posts' => true,
                    'edit_published_posts' => true,
                    'delete_posts' => true,
                    'delete_others_posts' => true,
                    'delete_published_posts' => true,
                    'upload_files' => true,
                ]
            );
        }

        $administrators = get_role('administrator');
        if (!$administrators->has_cap('business_manager_access')) {
            $administrators->add_cap('business_manager_access');
        }

        $bm_employees = get_role('bm_employee');
        if (!$bm_employees->has_cap('business_manager_access')) {
            $bm_employees->add_cap('business_manager_access');
        }
    }

    /**
     * Redirect users with the 'bm_employee' when required.
     * @since  1.4.0
     */
    public function bm_redirect()
    {
        global $pagenow;

        $user = get_current_user_id();
        $user_bm_employee = get_user_meta($user, 'bm_employee', true);
        $post = (isset($_GET['post']) ? sanitize_text_field($_GET['post']) : null);

        if (current_user_can('bm_employee')) {
            if ($pagenow == 'index.php') {
                wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                exit;
            }

            if (isset($_GET['post_type'])) {
                $post_type = sanitize_text_field($_GET['post_type']);
            }

            // Employees
            if ((isset($post_type) && $post_type == 'bm-employee') || (isset($post_type) && get_post_type($post) == 'bm-employee')) {
                if (business_manager_employee_access($user, 'bm_access_employees') == 'none') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                } elseif (business_manager_employee_access($user, 'bm_access_employees') == 'limited' && ($pagenow == 'post-new.php' || ($post !== null && $user_bm_employee != $post))) {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                }
            }

            // Leave
            if ((isset($post_type) && $post_type == 'bm-leave') || (isset($post_type) && get_post_type($post) == 'bm-leave')) {
                if (business_manager_employee_access($user, 'bm_access_leave') == 'none') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                } elseif (business_manager_employee_access($user, 'bm_access_leave') == 'limited' && ($post !== null && $user_bm_employee != get_post_meta($post, '_bm_leave_employee', true))) {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                }
            }

            // Reviews
            if ((isset($post_type) && $post_type == 'bm-review') || (isset($post_type) && get_post_type($post) == 'bm-review')) {
                if (business_manager_employee_access($user, 'bm_access_reviews') == 'none') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                } elseif (business_manager_employee_access($user, 'bm_access_reviews') == 'limited' && ($pagenow == 'post-new.php' || ($post !== null && $user_bm_employee != get_post_meta($post, '_bm_review_employee', true)))) {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                }
            }

            // Projects
            if ((isset($post_type) && $post_type == 'bm-project') || (isset($post_type) && get_post_type($post) == 'bm-project')) {
                if (business_manager_employee_access($user, 'bm_access_projects') == 'none') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                } elseif (business_manager_employee_access($user, 'bm_access_projects') == 'limited' && ($pagenow == 'post-new.php' || ($post !== null && $user_bm_employee != get_post_meta($post, '_bm_project_assigned_to', true)))) {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                }
            }

            // Clients
            if ((isset($post_type) && $post_type == 'bm-client') || (isset($post_type) && get_post_type($post) == 'bm-client')) {
                if (business_manager_employee_access($user, 'bm_access_clients') == 'none') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                } elseif (business_manager_employee_access($user, 'bm_access_clients') == 'limited' && $pagenow == 'post-new.php') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                }
            }

            // Documents
            if ((isset($post_type) && $post_type == 'bm-document') || (isset($post_type) && get_post_type($post) == 'bm-document')) {
                if (business_manager_employee_access($user, 'bm_access_documents') == 'none') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                } elseif (business_manager_employee_access($user, 'bm_access_documents') == 'limited' && $pagenow == 'post-new.php') {
                    wp_redirect(admin_url('admin.php?page=business-manager'), 301);
                    exit;
                }
            }
        }
    }

    /**
     * Add class to body tag.
     * @since  1.4.4
     */
    public function bm_body_class($classes)
    {
        global $post_type;

        if (!empty($post_type) && substr($post_type, 0, 3) == 'bm-') {
            $classes .= ' business-manager';
        }

        return $classes;
    }

    /**
     * Load Localisation files.
     * @since  1.0.0
     */
    public function localisation()
    {
        $locale = apply_filters('plugin_locale', get_locale(), 'business-manager');

        load_textdomain('business-manager', WP_LANG_DIR.'/business-manager/business-manager-'.$locale.'.mo');
        load_plugin_textdomain('business-manager', false, plugin_basename(dirname(__FILE__)).'/languages');
    }
}

/**
 * Run the plugin.
 */
function business_manager()
{
    return Business_Manager::instance();
}
business_manager();
