<?php

if (!defined('ABSPATH')) {
    exit;
}

if (!class_exists('Business_Manager_Employee', false)) :

class Business_Manager_Employee
{
    public $post_type = 'bm-employee';
    public $post_id = null;
    public $cmb = null;

    public $access_type = 'bm_access_employees';
    public $bm_employee_id;
    public $bm_access;

    public function __construct($post_id = null)
    {
        $this->post_id = $post_id;

        $this->init();
        $this->hooks();
    }

    /**
     * Init.
     *
     * @since 1.0.0
     */
    public function init()
    {
        global $post;

        // if we are in the post edit screen
        if (isset($post->ID) && $post->post_type == $this->post_type) {
            $this->post_id = (int)$post->ID;
        }
    }

    /**
     * Hook in to actions & filters.
     *
     * @since 1.4.0
     */
    public function hooks()
    {
        add_action('plugins_loaded', [$this, 'bm_pluggable']);

        // Interface Changes
        add_filter('gettext', [$this, 'publish_metabox_rename'], 10, 2);
        add_filter('post_updated_messages', [$this, 'post_updated_message']);
        add_action('admin_menu', [$this, 'login_access_admin_menu']);
        add_filter('views_edit-'.$this->post_type, [$this, 'login_access_views_edit'], 1);
        add_filter('post_row_actions', [$this, 'login_access_post_row_actions'], 10, 2);
        add_filter('bulk_actions-edit-'.$this->post_type, [$this, 'login_access_bulk_actions']);

        add_action('cmb2_save_post_fields', [$this, 'on_save_update'], 10, 4);
    }

    /**
    *
    * Set Business Manager Variables that rely on pluggable functions.
    *
    * @since 1.4.1
    */
    public function bm_pluggable()
    {
        $this->bm_employee_id = business_manager_employee_id(get_current_user_id());
        $this->bm_access = business_manager_employee_access(get_current_user_id(), $this->access_type);
    }

    /**
    *
    * Rename Publish text.
    *
    * @since 1.4.4
    */
    public function publish_metabox_rename($translation, $text)
    {
        global $post_type;

        if ($post_type == $this->post_type && ($text == 'Publish' || $text == 'Update')) {
            return __('Save Employee', 'business-manager');
        }

        return $translation;
    }

    /**
    *
    * Rename post updated message.
    *
    * @since 1.4.4
    */
    public function post_updated_message($messages)
    {
        global $post_type;

        if ($post_type == $this->post_type) {
            $messages['post'] = array_map(
                function ($string) {
                    return str_replace('Post', __('Employee', 'business-manager'), str_replace('updated', __('saved', 'business-manager'), str_replace('published', __('saved', 'business-manager'), $string)));
                },
                $messages['post']
            );
        }

        return $messages;
    }

    /**
     *
     * Change interface based on Employee Login Access.
     *
     * @since 1.4.0
     */
    public function login_access_admin_menu()
    {
        global $pagenow;

        $this->post_id = (isset($_GET['post']) ? sanitize_text_field($_GET['post']) : null);
        $post_type = (isset($_GET['post_type']) ? sanitize_text_field($_GET['post_type']) : get_post_type($this->post_id));

        if (current_user_can('manage_options') || $post_type != $this->post_type) {
            return;
        }

        // Hide Metaboxes
        if ($this->bm_access == 'limited' && $pagenow == 'post.php') {
            remove_meta_box('submitdiv', $this->post_type, 'side');
            remove_meta_box('_bm_employee_access_box', $this->post_type, 'normal');
        }

        // Hide Buttons
        if ($this->bm_access == 'limited') {
            echo '<style type="text/css">';
            echo 'a.page-title-action { display: none; }';
            echo '.cmb-add-row { display: none; }';
            echo '.cmb2-upload-button, .cmb2-remove-file-button { display: none !important; }';
            echo '</style>';
        }
    }

    /**
     *
     * Change view options based on Employee Login Access.
     *
     * @since 1.4.0
     */
    public function login_access_views_edit($views)
    {
        global $pagenow;

        $post_type = (isset($_GET['post_type']) ? sanitize_text_field($_GET['post_type']) : null);

        if (current_user_can('manage_options') || $this->bm_access == 'full' || $post_type != $this->post_type) {
            return $views;
        }

        $remove_views = ['all', 'publish', 'future', 'sticky', 'draft', 'pending', 'trash', 'mine'];

        foreach ((array) $remove_views as $view) {
            if (isset($views[$view])) {
                unset($views[$view]);
            }
        }

        return $views;
    }

    /**
     *
     * Change row actions based on Employee Login Access.
     *
     * @since 1.4.0
     */
    public function login_access_post_row_actions($actions, $post)
    {
        global $pagenow;

        $post_type = (isset($_GET['post_type']) ? sanitize_text_field($_GET['post_type']) : null);

        if (current_user_can('manage_options') || $post_type != $this->post_type) {
            return $actions;
        }

        if ($this->bm_access == 'limited') {
            if ($post->ID == $this->bm_employee_id) {
                $actions['edit'] = str_replace(__('Edit'), __('View'), $actions['edit']);
            } else {
                unset($actions['edit']);
            }

            unset($actions['trash'], $actions['inline hide-if-no-js']);
        }

        return $actions;
    }

    /**
     *
     * Change bulks actions based on Employee Login Access.
     *
     * @since 1.4.0
     */
    public function login_access_bulk_actions($actions)
    {
        global $pagenow;

        $post_type = (isset($_GET['post_type']) ? sanitize_text_field($_GET['post_type']) : null);

        if (current_user_can('manage_options') || $this->bm_access == 'full' || $post_type != $this->post_type) {
            return $actions;
        }
    }

    /**
     * Run after saving post.
     *
     * @since 1.0.0
     */
    public function on_save_update($object_id, $cmb_id, $updated, $cmb)
    {
        if (get_post_type($object_id) != $this->post_type) {
            return;
        }

        $this->post_id = $object_id;
        $this->cmb = $cmb;

        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }

        if (wp_is_post_revision($this->post_id)) {
            return;
        }

        remove_action('cmb2_save_post_fields', [$this, 'on_save_update'], 10, 4);

        if ($cmb_id == '_bm_employee_access_box' && $cmb->data_to_save['_bm_employee_user_id'] != 0) {
            // Remove this Business Manager Employer ID from all current Business Manager Users
            $users = get_users([
                'meta_key' => 'bm_employee',
                'meta_value' => $this->post_id,
            ]);

            foreach ($users as $u) {
                delete_user_meta($u->ID, 'bm_employee');
            }

            // Add this Business Manager Employer ID to the select Business Manager User
            update_user_meta($cmb->data_to_save['_bm_employee_user_id'], 'bm_employee', $this->post_id);
        }

        $this->update_title();

        add_action('cmb2_save_post_fields', [$this, 'on_save_update'], 10, 4);
    }

    /**
     * Update title.
     *
     * @since 1.0.0
     */
    public function update_title()
    {
        $name = business_manager_employee_full_name($this->post_id);
        if (!$name) {
            $name = __('(no name)', 'business-manager');
        }
        $post = [
            'ID' => $this->post_id,
            'post_title' => $name,
        ];
        wp_update_post($post);
    }

    /**
     * Returns data for all published employees.
     *
     * @since 1.0.0
     */
    public function get_employees($query_args = [])
    {
        $query_args['post_type'] = $this->post_type;
        $post_data = business_manager_post_items($query_args);

        return $post_data;
    }

    /**
     * HTML for the upcoming leave metabox
     * within employee edit screen.
     *
     * @since 1.0.0
     */
    public function upcoming_leave_html_metabox()
    {
        $leave = $this->get_leave_data();
        $count = 0;

        if (!empty($leave)) {
            foreach ($leave as $key => $data) {
                $meta = $data['meta'];
                $type = $meta['_bm_leave_type'][0];
                $status = $meta['_bm_leave_status'][0];
                $start = $meta['_bm_leave_start'][0];
                $end = $meta['_bm_leave_end'][0];
                $days = $meta['_bm_leave_total_days'][0];

                // skip if leave is in the past
                if (current_time('timestamp') > $start) {
                    continue;
                } ?>

				<div class="employee-leave bm-metabox">
					<div class="type">
						<span><?php _e('Type:', 'business-manager'); ?></span> 
						<span><?php echo esc_html($type); ?></span> 
					</div>
					<div class="status">
						<span><?php _e('Status:', 'business-manager'); ?></span>
						<span class="<?php echo esc_attr(strtolower($status)); ?>"><?php echo esc_html($status); ?></span> 
					</div>
					<div class="days">
						<span><?php _e('Total Days:', 'business-manager'); ?></span> 
						<span><?php echo esc_html($days); ?></span> 
					</div>
					<div class="start">
						<span><?php _e('First Day:', 'business-manager'); ?></span> 
						<span><?php echo business_manager_date_format($start); ?></span> 
					</div>
					<div class="end">
						<span><?php _e('Last Day:', 'business-manager'); ?></span> 
						<span><?php echo business_manager_date_format($end); ?></span> 
					</div>
					<a class="button button-secondary button-small" href="<?php echo get_edit_post_link($data['id']); ?>"><?php _e('Edit Leave', 'business-manager'); ?></a>
				</div>

			<?php
            $count++;
            }
        }

        if ($count == 0) {
            _e('No upcoming leave', 'business-manager');
        }
    }

    /**
     * HTML for the main employee column
     * on employee list table.
     *
     * @since 1.3.2
     */
    public function employee_html_column($link)
    {
        ?>
        <div class="bm-employee-column">
            <?php if ($link != '') : ?>
            <a href="<?php echo $link; ?>"><?php echo business_manager_employee_photo(get_post_meta($this->post_id, '_bm_employee_photo_id', true)); ?></a>
            <?php else : ?>
            <?php echo business_manager_employee_photo(get_post_meta($this->post_id, '_bm_employee_photo_id', true)); ?>
            <?php endif; ?>
        </div>

        <div class="bm-employee-column">
            <?php if ($link != '') : ?>
            <a href="<?php echo $link; ?>"><?php echo business_manager_employee_full_name($this->post_id) ; ?></a>
            <?php else : ?>
            <?php echo business_manager_employee_full_name($this->post_id) ; ?>
            <?php endif; ?>
        </div>
        <?php
    }

    /**
     * HTML for the upcoming leave column
     * on employee list table.
     *
     * @since 1.0.0
     */
    public function upcoming_leave_html_column()
    {
        $leave = $this->get_leave_data();

        if (!empty($leave)) {
            foreach ($leave as $key => $data) {
                $meta = $data['meta'];
                $type = $meta['_bm_leave_type'][0];
                $status = $meta['_bm_leave_status'][0];
                $start = $meta['_bm_leave_start'][0];
                $end = $meta['_bm_leave_end'][0];
                $days = $meta['_bm_leave_total_days'][0];

                // skip if leave is in the past
                if (current_time('timestamp') > $start) {
                    continue;
                } ?>

				<div class="employee-leave columns">

					<div class="type">
						<span><?php echo esc_html($days); ?> <?php _e('Days', 'business-manager'); ?> - <?php echo esc_html($type); ?></span> 
					</div>
					<div class="status">
						<span class="<?php echo esc_attr(strtolower($status)); ?>"><?php echo esc_html($status); ?></span> 
					</div>
					<div class="dates">
						<span><?php echo business_manager_date_format($start); ?> - <?php echo business_manager_date_format($end); ?></span> 
					</div>

				</div>

			<?php
            }
        }
    }

    /**
     * HTML for the upcoming leave on the dashboard.
     *
     * @since 1.0.0
     */
    public function upcoming_leave_html_dashboard()
    {
        $leave = $this->get_leave_data();
        $count = 0;

        if (!empty($leave)) {
            foreach ($leave as $key => $data) {
                $meta = $data['meta'];
                $id = $meta['_bm_leave_employee'][0];
                $type = $meta['_bm_leave_type'][0];
                $status = $meta['_bm_leave_status'][0];
                $start = $meta['_bm_leave_start'][0];
                $end = $meta['_bm_leave_end'][0];
                $days = $meta['_bm_leave_total_days'][0];

                // skip if leave is in the past
                if (current_time('timestamp') > $start) {
                    continue;
                } ?>

				<div class="bm-dashboard-row">
                    <div class="bm-dashboard-column image">
                        <?php echo business_manager_employee_photo(get_post_meta($id, '_bm_employee_photo_id', true)); ?>
                    </div>

                    <div class="bm-dashboard-column">
                        <div class="name">
                            <a href="<?php echo get_edit_post_link($data['id']); ?>"><?php echo business_manager_employee_full_name($id) ?></a>
                        </div>
                        
                        <div class="type">
                            <?php echo esc_html($days); ?> <?php _e('Days', 'business-manager'); ?> - <?php echo esc_html($type); ?>
                        </div>
                        
                        <div class="status">
                            <span class="<?php echo esc_attr(strtolower($status)); ?>"><?php echo esc_html($status); ?></span> 
                        </div>
                        
                        <div class="dates">
                            <?php echo business_manager_date_format($start); ?> - <?php echo business_manager_date_format($end); ?>
                        </div>
                    </div>
				</div>

			<?php
            $count++;
            }
        }

        if ($count == 0) {
            _e('No upcoming leave', 'business-manager');
        }
    }

    /**
     * Get all leave for an employee (or all employees)
     * including past and future leave.
     *
     * @since 1.0.0
     */
    public function get_leave_data()
    {
        $user = get_current_user_id();
        $user_bm_employee = get_user_meta($user, 'bm_employee', true);

        $query = [
            'key' => '_bm_leave_employee',
        ];

        if (isset($user_bm_employee) && !empty($user_bm_employee) && business_manager_employee_access($user, 'bm_access_leave') == 'limited') {
            $query['value'] = $user_bm_employee;
        }

        if (is_int($this->post_id) && $this->post_id != null) {
            $query['value'] = $this->post_id;
            $query['compare'] = '=';
        }

        $query_args = [
            'post_status' => 'publish',
            'post_type' => 'bm-leave',
            'posts_per_page' => -1,
            'meta_query' => [
                $query,
            ],
        ];

        $data = business_manager_post_items($query_args);

        return $data;
    }

    public function get_next_birthday($birthday)
    {
        $date = new DateTime($birthday);
        $date->modify('+'.date('Y') - $date->format('Y').' years');
        if ($date < new DateTime()) {
            $date->modify('+1 year');
        }

        return $date->format('Y-m-d');
    }

    /**
     * HTML for the upcoming birthdays on the dashboard.
     *
     * @since 1.0.0
     */
    public function upcoming_birthdays_html_dashboard()
    {
        $args_meta = [
            [
                'key' => '_bm_employee_status',
                'value' => 'Active',
            ],
        ];

        $args = [
            'post_status' => 'publish',
            'post_type' => $this->post_type,
            'posts_per_page' => -1,
            'meta_key' => '_bm_employee_dob',
            'meta_query' => $args_meta,
            'orderby' => 'meta_value_num',
            'order' => 'ASC',
        ];

        $posts = get_posts($args);
        $count = 0;

        if (!empty($posts)) {
            $upcoming_birthdays = [];

            foreach ($posts as $key => $post) {
                $post->birthday = $this->get_next_birthday(date('Y-m-d', get_post_meta($post->ID, '_bm_employee_dob', true)));

                // Exclude Employees with birthday more than 3 months away from today
                if (strtotime($post->birthday) >= strtotime('now') + (3 * MONTH_IN_SECONDS)) {
                    unset($posts[$key]);
                }
            }

            // Sort birthdays, soonest to latest
            usort($posts, function ($a, $b) { return strcmp($a->birthday, $b->birthday); });

            foreach ($posts as $key => $post) {
                $dob = get_post_meta($post->ID, '_bm_employee_dob', true);
                $dob = date('Y-m-d', $dob);
                $birthday = $this->get_next_birthday($dob); ?>

				<div class="bm-dashboard-row">
                    <div class="bm-dashboard-column image">
                            <?php echo business_manager_employee_photo(get_post_meta($post->ID, '_bm_employee_photo_id', true)); ?>
                        </div>

                    <div class="bm-dashboard-column">
                        <div class="name">
                            <a href="<?php echo get_edit_post_link($post->ID); ?>"><?php echo business_manager_employee_full_name($post->ID) ?></a><br>
                            <span><?php echo business_manager_employee_job_title($post->ID) ?></span> 
                        </div>

                        <div class="date">
                            <?php echo business_manager_date_format(strtotime($birthday)); ?>
                        </div>
                    </div>
				</div>

			<?php
            $count++;
            }
        }

        if ($count == 0) {
            _e('No upcoming birthdays', 'business-manager');
        }
    }
}

endif;

return new Business_Manager_Employee();
